/* Utilities for spawning sub-processes
 *
 * Copyright (c) 2018-2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef __SPAWN_H__
#define __SPAWN_H__

#include <stdbool.h>
#include <sys/types.h>

#ifdef __cplusplus
extern "C" {
#endif

#define DEFAULT_COMMAND_TIMEOUT_MS (60*1000) /* 60sec */

struct spawn_param;

typedef int (*spawn_prepare_fn)(struct spawn_param *param);

typedef struct spawn_param {
	spawn_prepare_fn fn;

	struct spawn_param *next;
	union {
		int int_val;
		mode_t mode_t_val;
		char *char_ptr;
	} u;
} spawn_param_s;

int spawn_setstdout(spawn_param_s *param);
int spawn_setstderr(spawn_param_s *param);
int spawn_nullstdfds(spawn_param_s *param);
int spawn_chdir(spawn_param_s *param);
int spawn_umask(spawn_param_s *param);

bool wait_for_pid(pid_t pid, int *exit_code);
bool spawn(char *const av[], char *const ev[], spawn_param_s *param, pid_t *childpid, int *childfd);
bool spawn_wait(char *const av[], char *const ev[], spawn_param_s *param, int timeout_ms, int *exit_code);

#ifdef __cplusplus
}
#endif

#endif
